require_dependency "skirental_frontend_api/application_controller"

module SkirentalFrontendApi
    class BookingsController < ApplicationController

        def context
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first
            if booking.present? and booking.context.present?
                booking.context[:changeableUntil] = booking.changeable_until
            end
            render json: booking
        end

        def create
            # check whether booking was already created. This should not happen, but..
            new_booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first
            if new_booking.nil?
                new_booking = Booking.new(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id], context: booking_params[:booking].to_h)
                if booking_params[:creation_time].present?
                    new_booking.creation_time = booking_params[:creation_time]
                end
            else
                # TODO: should be a fail!
                new_booking.context = booking_params[:booking].to_h
            end
            if new_booking.save
                render json: { error: "" }, :status => :created
            else
                render json: { error: new_booking.errors.full_messages }, :status => :unprocessable_entity
            end
        end

        def updatability
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first
            api_booking = SkirentalApi::Client.get_booking(booking.booking_id)
            if (api_booking[:editUntil] != booking.changeable_until)
                booking.changeable_until = api_booking[:editUntil]
                booking.no_sync = true
                booking.save
            end
            puts "***\n***\n***\nBOOKING UPDATABILITY for id '#{booking.booking_id}': #{booking.changeable_until}\n***\n***\n***"
            render json: { changeable_until: api_booking[:editUntil] }
        end

        def update
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first

            if booking.nil?
                render json: { error: "not found" }, :status => :not_found
            else
                unless booking.already_finished
                    puts "BOOKING FOUND, TRYING TO SAVE UPDATED CONTEXT"
                    booking.context = booking_params[:booking].to_h
                    puts "#{booking.context}"
                    if booking.save
                        render json: { error: "", booking_id: booking.booking_id, voucher: booking.voucher }, :status => :ok
                    else
                        render json: { error: booking.errors.full_messages }, :status => :unprocessable_entity
                    end
                else
                    # don't allow everything -> only customer.personalData and product.selectedVariant can be updated from now on…
                    puts "BOOKING ALREADY FINISHED!!"
                    begin
                        # TODO: Update for new structure
                        updated_context = booking_params[:booking].to_h

                        # update booking.code once if not set yet (directly after payment)
                        if booking.context["code"].blank? and updated_context["code"].present?
                            booking.context["code"] = updated_context["code"]
                        end

                        # update customer notice
                        booking.context["customerNotice"] = updated_context["customerNotice"]

                        # update followedEmailLink variable
                        booking.context["followedEmailLink"] = updated_context["followedEmailLink"]
        
                        # Iterate over updated customers
                        updated_context["customers"].each_with_index do |customer, customer_index|
                            # Overwrite personalData in persisted context
                            booking.context["customers"][customer_index]["personalData"] = customer["personalData"]

                            # Overwrite selectedVariants in persisted context
                            customer["packages"].each_with_index do |package, package_index|
                                booking.context["customers"][customer_index]["packages"][package_index]["mainProduct"]["selectedVariant"] = package["mainProduct"]["selectedVariant"]
                                package["accessories"].each_with_index do |accessorie, accessorie_index|
                                    booking.context["customers"][customer_index]["packages"][package_index]["accessories"][accessorie_index]["selectedVariant"] = accessorie["selectedVariant"]
                                end
                            end
                            customer["products"].each_with_index do |product, product_index|
                                booking.context["customers"][customer_index]["products"][product_index]["selectedVariant"] = product["selectedVariant"]
                            end
                        end

                        if booking_params[:no_sync].to_s.present?
                            if booking_params[:no_sync].to_s == 'true'
                                booking.no_sync = true
                            end
                        end

                        if booking.save
                            render json: { error: "", booking_id: booking.booking_id, voucher: booking.voucher }, :status => :ok
                        else
                            render json: { error: booking.errors.full_messages }, :status => :unprocessable_entity
                        end
                    rescue Exception => e
                        # most probably the lengths of customers and/or products arrays didn't match…
                        render json: { error: "already paid or reserved: #{e}" }, :status => :unprocessable_entity
                    end
                end
            end
        end

        def accept
            booking = Booking.where(shop_id: booking_params[:shop_id], session_id: booking_params[:session_id]).first

            if booking.context["code"].present?
                response = SkirentalApi::Client.accept_booking(booking.context["code"])
                render json: { error: nil, response: response }, :status => :ok
            else
                render json: { error: "no booking code available" }, :status => :unprocessable_entity
            end
        end

        private

        def booking_params
            params.permit(:shop_id, :session_id, :no_sync, :creation_time, booking: {})
        end
    end
end
